@@ -416,7 +416,7 @@ local function body_reader(s, headers)
             end
 
             if not body then
-                if partial and #partial > 0 then
+                if partial then
                     log.err(string.format('with %d bytes remaining to read: ' .. err, content_length - #partial))
                     return partial
                 end
@@ -659,6 +659,10 @@ function con_methods:send_error(code, status, content)
         return false, 'closed'
     end
 
+    if type(code) ~= 'number' then
+        error('invalid code: ' .. tostring(code))
+    end
+
     self:set_status(code, status)
     self:add_header('connection', 'close')
 
@@ -732,12 +736,9 @@ end
 
 function con_methods:send_file_fd(fd, size, count, offset)
     local mt = getmetatable(self)
-    local options = mt.options
     local resp = mt.resp
     local sock = mt.sock
 
-    assert(not options.ssl, 'not support sendfile for https')
-
     if count and count < 1 then
         return true
     end
@@ -764,9 +765,6 @@ end
 
 function con_methods:send_file(path, count, offset)
     local mt = getmetatable(self)
-    local options = mt.options
-
-    assert(not options.ssl, 'not support sendfile for https')
 
     if mt.sock:closed() then
         return false, 'closed'
@@ -790,10 +788,14 @@ function con_methods:send_file(path, count, offset)
         return false, err
     end
 
-    local ok, err = self:send_file_fd(fd, st.size, count, offset)
+    local _, err = self:send_file_fd(fd, st.size, count, offset)
     file.close(fd)
 
-    return ok, err
+    if err then
+        return false, err
+    end
+
+    return true
 end
 
 function con_methods:flush()
@@ -830,6 +832,10 @@ function con_methods:read_body(count, timeout)
         return nil, 'closed'
     end
 
+    if body_remain == 0 then
+        return nil
+    end
+
     timeout = timeout or 3.0
 
     count = count or body_remain
@@ -841,9 +847,13 @@ function con_methods:read_body(count, timeout)
         return ''
     end
 
-    local data, err = sock:recv(count, timeout)
+    local data, err, partial = sock:recvfull(count, timeout)
     if not data then
         sock:close()
+
+        if partial then
+            return nil, err, partial
+        end
         return nil, err
     end
 
@@ -953,29 +963,14 @@ function con_methods:serve_file(req)
         return self:send_error(M.STATUS_INTERNAL_SERVER_ERROR, nil, string.format('open "%s" fail: %s', phy_path, err))
     end
 
-    local ok, err, data
-
-    if options.ssl then
-        while true do
-            data, err = file.read(fd, 4096)
-            if not data then
-                break
-            end
-
-            if #data == 0 then
-                ok = true
-                break
-            end
-
-            self:send(data)
-        end
-    else
-        ok, err = self:send_file_fd(fd, st.size)
-    end
-
+    local _, err = self:send_file_fd(fd, st.size)
     file.close(fd)
 
-    return ok, err
+    if err then
+        return false, err
+    end
+
+    return true
 end
 
 local function handle_connection(con, peer, handler)
@@ -1152,10 +1147,6 @@ local function set_socket_options(sock, options)
         sock:setoption('tcp_keepintvl', tcp_keepalive)
         sock:setoption('tcp_fastopen', 5)
     end
-
-    if options.ipv6 then
-        sock:setoption('ipv6_v6only', true)
-    end
 end
 
 function M.listen(ipaddr, port, options, handler)
@@ -1175,12 +1166,14 @@ function M.listen(ipaddr, port, options, handler)
     if options.cert and options.key then
         options.ssl = true
         if options.ipv6 then
+            options.ipv6_v6only = true
             listen = ssl.listen6
         else
             listen = ssl.listen
         end
     else
         if options.ipv6 then
+            options.ipv6_v6only = true
             listen = socket.listen_tcp6
         else
             listen = socket.listen_tcp
@@ -1219,7 +1212,7 @@ function M.listen(ipaddr, port, options, handler)
                 end
             end, c)
         else
-            log.err('accept fail: ' .. err)
+            log.err('accept fail: ' .. peer)
         end
     end
 end
